from scipy.interpolate import interp1d
import numpy

def getAirGapLength(diameter_from_main_dimension, air_gap_matrix):
    ans_air_gap = 0
    dia_column = air_gap_matrix[:,0]
    air_gap_column = air_gap_matrix[:,1]
    if diameter_from_main_dimension <= 0.15:
        ans_air_gap = 0.3
    elif diameter_from_main_dimension >= 0.8:
        ans_air_gap = 4
    else:
        f = interp1d(dia_column, air_gap_column)
        ans_air_gap = f(diameter_from_main_dimension).min()
    return ans_air_gap


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from math import pi
from ..classes.metric import Length, Area

def stator_slot_leakage(Ws, W1, W0, isCircular, h1, h3, h4):
    print(W1.m, W0.m, Ws.m)
    if isCircular:
        sp_slot_per = 4*pi*pow(10,-7)*(2*h1.m/(3*(Ws.m + W1.m))+2*h3.m/(W1.m + W0.m) + h4.m/W0.m)
    else:
        sp_slot_per = 4*pi*pow(10,-7)*(2*h1.m/(3*Ws.m) + 2*h3.m/(Ws.m + W0.m)+ h4.m/W0.m)
    
    print("Specific slot permeance for stator: ", sp_slot_per)    
    return sp_slot_per



import os
import pandas
import numpy
from scipy.interpolate import interp1d

package_dir = os.path.dirname(os.path.abspath(__file__))

carter_coeff = pandas.read_excel(package_dir+'/lookupTables/Carter.xlsx')
carter_matrix = carter_coeff.values

def getCarterCoeff(typeOfSlot, ratio):
    ans_carter = 0
    if typeOfSlot == 'semiClosed':
        semi_closed_column = carter_matrix[:,1]
        ratio_column = carter_matrix[:,0]
        if ratio <= 0:
            ans_carter = 0
        elif ratio >= 12:
            ans_carter = 0.95
        else:
            f = interp1d(ratio_column, semi_closed_column)
            ans_carter = f(ratio).min()
    elif typeOfSlot == 'open':
        open_column = carter_matrix[:,2]
        ratio_column = carter_matrix[:,0]
        if ratio <= 0:
            ans_carter = 0
        elif ratio >= 12:
            ans_carter = 0.72
        else:
            f = interp1d(ratio_column, open_column)
            ans_carter = f(ratio).min()
    else:
        print("Mayday...")
    return ans_carter

print(getCarterCoeff('semiClosed', 4.8))


class Length:
    m = 0
    mm = 0
    def __init__(self, length_in_meter):
        self.m = length_in_meter
        self.mm = pow(10,3)*length_in_meter
    def setLength(self, length_in_meter):
        self.m = length_in_meter
        self.mm = pow(10,3)*length_in_meter

class Area:
    m2 = 0
    mm2 = 0
    def __init__(self, area_in_meter_square):
        self.m2 = area_in_meter_square
        self.mm2 = pow(10,6)*area_in_meter_square
    def setArea(self, area_in_meter_square):
        self.m2 = area_in_meter_square
        self.mm2 = pow(10,6)*area_in_meter_square



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from ..classes.metric import Length,Area
from ..noLoadCurrent.LohysBAt import getATFromLohys
from ..rating import poles 
from math import pi

def mag_rotor_core(Bcr, Li, Dr, dsr, dcs, dcr, lohys_bat_matrix):
    Acr = Area(0)
    lcr = Length(0)
    
    Acr.setArea(Li.m*dcr.m)
    print("Area of rotor core: ", Acr.m2)
    
    print("Flux density in rotor core: ", Bcr)
    
    alpha = getATFromLohys(Bcr, lohys_bat_matrix)
    print("alpha_tsr: ", alpha)
    
    lcr.setLength(pi*(Dr.m - 2*dsr.m - dcs.m)/(3*poles))
    print("Length of flux path in rotor core: ", lcr.m)
    
    ATcr = alpha*lcr.m
    print("Mmf required for rotor core: ", ATcr)
    
    return ATcr



from . import rating
from . import loadTable
import numpy
import pandas
import importlib
from .classes.metric import Length, Area
from .mainDimensions.mainDimensions import compute_main_dimension
from .statorDesign.statorWinding import stator_design_winding
from .statorDesign.adjustment import adjust_initial_values
from .statorDesign.statorConductorSlot import stator_conductor_size_and_slot
from .statorDesign.statorCore import stator_core
from .rotorDesign.airGap import air_gap
from .rotorDesign.rotorSlot import rotor_slots
from .rotorDesign.endRings import end_rings_rotor
from .rotorDesign.rotorCore import rotor_core
from .noLoadCurrent.magAirGap import mag_air_gap
from .noLoadCurrent.magStatorTeeth import mag_stator_teeth
from .noLoadCurrent.magStatorCore import mag_stator_core
from .noLoadCurrent.magRotorTeeth import mag_rotor_teeth
from .noLoadCurrent.magRotorCore import mag_rotor_core
from .noLoadCurrent.magNoLoad import mag_no_load
from .noLoadCurrent.lossStatorTeeth import loss_stator_teeth
from .noLoadCurrent.lossStatorCore import loss_stator_core
from .noLoadCurrent.noLoadCurrent import no_load_current
from .shortCircuitCurrent.statorSlotLeakage import stator_slot_leakage
from .shortCircuitCurrent.rotorSlotLeakage import rotor_slot_leakage
from .shortCircuitCurrent.slotLeakageReactance import slot_leakage_reactance
from .shortCircuitCurrent.overhangLeakage import overhang_leakage
from .shortCircuitCurrent.totalLeakage import total_leakage
from .shortCircuitCurrent.Resistance import resistance
from .shortCircuitCurrent.impedanceLossesEfficiency import Z_losses_eff
from .temperatureRise.temperatureRise import temperature_rise

def design_your_machine():
    swg_matrix, rect_matrix, carter_matrix, lohys_bat_matrix, fwloss_matrix, lohys_loss_matrix, air_gap_matrix, slot_leakage_matrix = loadTable.load_table()
    print()
    print()
    print('Hello!')
    print()
    print()

    # Variables
    Bav = 0.42
    Kw = 0.955
    ac = 20000
    qs = 2
    L_by_tau = 1.6
    delta = 6
    space_factor = 0.4

    count = 0
    currentCriterion = False
    while 1 - (currentCriterion  == True and Bav >= 0.35):
        Bav = 0.42
        # Enter Bav, ac, Kw, L/tau ratio
        D, L, tau, Li, isDuct, C0, Q, D2L = compute_main_dimension(Bav, ac, Kw, L_by_tau)
        # D, L, tau, Li, isDuct = compute_main_dimension(0.65, 20000, 0.95, 1.5)
        print('===============================================================')
        #print(D.m, L.m, Li.m, tau.m, isDuct)

        # Continue from here
        phi, Ts, Ss, yss, Zss, Kws, Cs, Kp, Kd = stator_design_winding(Bav, tau.m, L.m, D.m, Kw, qs)

        # phi, Ts, Ss, yss, Zss, Kws = stator_design_winding(0.65, 0.112, 0.167, 0.214, 0.95, 2)

        print(phi, Ts, Ss, yss.mm, Zss, Kws)

        phi, Ts, Ss, Zss, Bav, ac = adjust_initial_values(Zss, Ss, qs, Bav, ac, tau, L, Kw)

        Is, W2, Wsss, dss, Wts, Lmts, isCircular, h4, h3, h1, Bts, As, a, b, delta_s = stator_conductor_size_and_slot(phi, Ss, rating.poles, D, L, Li, tau, delta, Zss, space_factor, rect_matrix, swg_matrix)

        Acs, dcs, D0, Bcs = stator_core(phi, Li, D, dss)

        print('Stator Design Completed')
        print('===============================================================')

        lg, Dr = air_gap(L, D, air_gap_matrix)

        Sr, Ib, rotor_bar_area, Wsr, dsr, Btr_13, rotor_copper_loss, Lb, ysr, W0, Wsrr, h1, h3, h4, noAreaFound, delta_r, Rb, a_r, b_r = rotor_slots(Ss, Kws, Ts, Is, Dr, rect_matrix, phi, Li, L)

        total_rotor_copper_loss, slip, Ae, delta_e, De, Re, Ie, ring_copper_loss = end_rings_rotor(Sr, Ib, Dr, dsr, rotor_copper_loss)

        dcr, Bcr, Di = rotor_core(Dr, dcs, dsr, Bcs)

        print('Rotor Design Completed')
        print('===============================================================')

        s_W0 = Length(0.002)
        r_W0 = Length(0.0015)
        ATg = mag_air_gap(s_W0, r_W0, lg, yss, ysr, isDuct, rating.duct_width, L, D, 1, Bav, carter_matrix)
        ATs = mag_stator_teeth(Ss, Wts, Li, Bts, dss, lohys_bat_matrix)
        ATcs = mag_stator_core(D, Li, dcs, dss, Bcs, lohys_bat_matrix)
        ATr = mag_rotor_teeth(Btr_13, dsr, lohys_bat_matrix)
        ATcr = mag_rotor_core(Bcr, Li, Dr, dsr, dcs, dcr, lohys_bat_matrix)
        Im, AT60 = mag_no_load(Kws, Ts, ATg, ATs, ATcs, ATcr, ATr)
        loss_st = loss_stator_teeth(Ss, Wts, Li, dss, Bts, lohys_loss_matrix)
        loss_sc = loss_stator_core(D0, D, dss, Li, Bcs, lohys_loss_matrix)
        I0, phase_angle, currentCriterion, total_noload_loss, fw_loss, iron_loss, Il, no_load_pf = no_load_current(loss_st, loss_sc, Im, Is, fwloss_matrix)

        print('No Load Current Completed')
        print('===============================================================')

        slot_permeance_stator = stator_slot_leakage(Wsss, W2, W0, isCircular, h1, h3, h4)
        slot_permeance_rotor = rotor_slot_leakage(Wsss, W0, h1, h3, h4, Kws, Ss, Sr)
        X0 = overhang_leakage(yss, tau, Cs, Ss, Ts, qs, slot_leakage_matrix)
        xs = slot_leakage_reactance(slot_permeance_stator, slot_permeance_rotor, Ts, L, qs)
        Xm, Xs, Xz = total_leakage(Im, Ss, Sr, xs, X0)
        Rs, total_stator_copper_loss, rs, Rrs = resistance(Ts, As, Lmts, Is, total_rotor_copper_loss)
        Z, Isc, pf_sc, phase_sc, loss_fl, eff = Z_losses_eff(Rs, Xs, total_stator_copper_loss, total_rotor_copper_loss, total_noload_loss)

        print('Short Circuit Current Completed')
        print('===============================================================')

        theta = temperature_rise(iron_loss, total_stator_copper_loss, L, Lmts, D, D0, 80)

        print('Temperature Rise Completed')
        print('===============================================================')

        print("Bav                      : ", round(Bav,3))
        print("AC                       : ", ac)
        print("Efficiency (expected)    : ", 100*rating.efficiency)
        print("Efficiency (obtained)    : ", round(eff,1))
        print("Temperature Rise (degC)  : ", round(theta))
        print("Current Criterion        : ", currentCriterion)
        if ac >= 18000 and ac <= 24000 and currentCriterion and Bav >= 0.35 and Bav <= 0.6:
            break
        else:
            count += 10
            # print("Exit count: ", count)
            if count >= 6000:
                print("EXIT")
                if qs == 3:
                    if ac >= 24000 and ac <= 18000:
                        count = 0
                        continue
                    else:
                        break 
                if qs == 2:
                    qs = 3
                    count = 0 
            else:
                ac = 18000 + count
                # print(ac, Bav)
    print('===============================================================')
    if currentCriterion:
        print("Preparing design sheet...")
        mat = []
        mat.append(["RATING", "", str(""), ""])
        mat.append(["Full Load Output ", "", str(rating.power_in_kw), "kW"])
        mat.append(["Line Voltage", "", str(rating.line_voltage), "V"])
        mat.append(["Frequency", "f", str(rating.frequency), "Hz"])
        mat.append(["Phases", "", 3, ""])
        mat.append(["Efficiency", "eta", str(rating.efficiency) , ""])
        mat.append(["Power Factor", "cos(phi)", str(rating.power_factor), "lag"])
        mat.append(["No. of Poles", "p", str(rating.poles) , ""])
        mat.append(["Synchronous Speed", "ns", str(rating.rpm), "rpm"])
        mat.append(["KVA input", "", str(round(Q, 3)), "KVA"])
        mat.append(["Full load line current", "", str(round(Is, 3)), "A"])
        mat.append(["LOADING", "", str(""), ""])
        mat.append(["Specific Magnetic Loading", "Bav", str(round(Bav,3)), "Wb/m^2"])
        mat.append(["Specific Electric Loading", "ac", str(round(ac)), "A/m"])
        mat.append(["Output Coefficient", "", str(round(C0, 3)), ""])
        mat.append(["D2L", "", str(round(D2L,5)), "m^3"])
        mat.append(["MAIN DIMENSIONS", "", str(""), ""])
        mat.append(["Stator Bore", "D", str(round(D.mm, 1)), "mm"])
        mat.append(["Gross Iron Length", "L", str(round(L.mm, 1)), "mm"])
        mat.append(["Ducts", "nd", str(int(isDuct)), ""])
        mat.append(["Net Iron Length", "Li", str(round(Li.mm, 1)), "mm"])
        mat.append(["Pole pitch", "tau", str(round(tau.mm, 1)), "mm"])
        mat.append(["STATOR", "", str(""), ""])
        mat.append(["Type of Lamination", "", str("0.5mm thick Lohys"), ""])
        mat.append(["Type of Winding", "", str("Single Layer Mush"), ""])
        mat.append(["Connection", "", str("delta"), ""])
        mat.append(["Phase Voltage", "Eb", str(rating.line_voltage), "V"])
        mat.append(["Flux per pole", "phi_m", str(round(phi, 6)), "Wb"])
        mat.append(["Turns per phase", "Ts", str(Ts), ""])
        mat.append(["Number of Slots", "Ss", str(Ss), ""])
        mat.append(["Slots per pole", "", str(3*int(qs)), ""])
        mat.append(["Slots per pole per phase", "qs", str(qs), ""])
        mat.append(["Coil Span", "Cs", str(int(Cs)), "slots"])
        mat.append(["Distribution Factor", "Kd", str(round(Kd, 3)), ""])
        mat.append(["Pitch Factor", "Kp", str(round(Kp, 3)), ""])
        mat.append(["Winding Factor", "Kws", str(round(Kws, 3)), ""])
        mat.append(["Slot Pitch", "yss", str(round(yss.mm, 1)), "mm"])
        mat.append(["Conductors per slot", "Zss", str(Zss), ""])
        if isCircular:
            mat.append(["Conductor: bare diameter", "", str(round(a, 4)), "mm"])
            mat.append(["Conductor: insulated diameter", "", str(round(a+0.025, 4)), "mm"])
        else:
            mat.append(["Conductor: bare length", "", str(round(a, 4)), "mm"])
            mat.append(["Conductor: bare breadth", "", str(round(b, 4)), "mm"])
            mat.append(["Conductor: insulated length", "", str(round(a+0.025, 4)), "mm"])
            mat.append(["Conductor: insulated breadth", "", str(round(b+0.025, 4)), "mm"])
        mat.append(["Conductor: area", "", str(As.mm2), "mm^2"])
        mat.append(["Current density", "", str(round(delta_s,2)), "A/mm^2"])
        mat.append(["Length of mean turn", "Lmts", str(round(Lmts.m, 2)), "m"])
        mat.append(["Phase resistance at 25degC", "rs", str(round(rs,2)), "ohm"])
        mat.append(["Copper Loss at Full Load", "3I_s^2r_s", str(round(total_stator_copper_loss, 1)), "W"])
        mat.append(["Depth of stator core", "dcs", str(round(dcs.mm)), "mm"])
        mat.append(["Outer diameter of stator laminations", "De", str(round(D0.mm, 1)), "mm"])
        mat.append(["ROTOR", "", str(""), ""])
        mat.append(["Length of air gap", "la", str(round(lg.mm, 2)), "mm"])
        mat.append(["Diameter of rotor", "Dr", str(round(Dr.mm, 1)), "mm"])
        mat.append(["Type of winding", "", str("Squirrel Cage"), ""])
        mat.append(["Number of slots", "Sr", str(Sr), ""])
        mat.append(["Slots per pole per phase", "qr", str(round(Sr/(rating.phase*rating.poles), 3)), ""])
        mat.append(["Conductors per slot", "Zsr", str(1), ""])
        mat.append(["Winding factor", "Kwr", str(1), ""])
        mat.append(["Slot pitch", "ysr", str(round(ysr.mm,2)), "mm"])
        mat.append(["Rotor bar current", "Ib", str(round(Ib, 2)), "A"])
        mat.append(["Rotor bar: cross-section", "", str(a_r)+"x"+str(b_r), "mm^2"])
        mat.append(["Rotor bar: area", "ab", str(rotor_bar_area.mm2), "mm^2"])
        mat.append(["Rotor bar: length", "Lb", str(round(Lb.mm, 1)), "mm"])
        mat.append(["Rotor bar: current density", "delta_b", str(round(delta_r, 2)), "A/mm^2"])
        mat.append(["Resistance of each bar", "Rb", str(round(Rb*pow(10,6), 2))+" x 10^(-6)", "ohm"])
        mat.append(["Copper loss in bars", "S_r*I_b^2*r_b", str(round(rotor_copper_loss, 1)), "W"])
        mat.append(["End ring current", "Ie", str(round(Ie, 1)), "A"])
        mat.append(["End ring: cross-section", "", "10 x "+str(Ae.mm2//10), "mm^2"])
        mat.append(["End ring: area", "", str(Ae.mm2), "mm^2"])
        mat.append(["End ring: mean diameter", "De", str(round(De.mm, 1)), "mm"])
        mat.append(["End ring: current density", "delta_e", str(round(delta_e, 2)), "A/mm^2"])
        mat.append(["Resistance of each ring", "Re", str(round(Re*pow(10,6), 2))+" x 10^(-6)", "ohm"])
        mat.append(["Copper loss in end rings", "2*I_e^2*R_e", str(round(ring_copper_loss, 1)), "W"])
        mat.append(["Total rotor copper loss", "", str(round(total_rotor_copper_loss, 1)), "W"])
        mat.append(["Resistance of rotor (referred to stator)", "Rr", str(round(Rrs, 2)), "ohm"])
        mat.append(["Depth of rotor core", "dcr", str(dcr.mm), "mm"])
        mat.append(["NO LOAD CURRENT", "", str(), ""])
        mat.append(["Magnetizing mmf per pole", "", str(round(AT60, 1)), "A"])
        mat.append(["Phase magnetizing current", "Im", str(round(Im,1 )), "A"])
        mat.append(["Magnetising reactance", "Xm", str(round(Xm, 1)), "ohm"])
        mat.append(["Core loss", "", str(round(iron_loss, 1)), "W"])
        mat.append(["Friction and Windage Loss", "", str(round(fw_loss,1)), "W"])
        mat.append(["No load loss", "", str(round(total_noload_loss, 1)), "W"])
        mat.append(["Loss Component", "Il", str(round(Il, 3)), "A"])
        mat.append(["No load current (phase)", "Is", str(round(I0, 3)), "A"])
        mat.append(["No load current (line)", "", str(round(1.732*I0, 3)), "A"])
        mat.append(["No load power factor", "cos(phi_0)", str(round(no_load_pf, 3)), "lag"])
        mat.append(["SHORT CIRCUIT CURRENT", "", str(""), ""])
        mat.append(["Slot Leakage Reactance", "xs", str(round(xs, 3)), "ohm"])
        mat.append(["Overhang Leakage Reactance", "x0", str(round(X0, 3)), "ohm"])
        mat.append(["Zigzag Leakage Reactance", "xz", str(round(Xz, 3)), "ohm"])
        mat.append(["Total Leakage Reactance", "Xt", str(round(Xs, 3)), "ohm"])
        mat.append(["Total Resistance", "Rs", str(round(Rs, 3)), "ohm"])
        mat.append(["Short Circuit Impedance", "Zs", str(round(Z, 3)), "ohm"])
        mat.append(["Phase Short Circuit Current", "Isc", str(round(Isc, 3)), "A"])
        mat.append(["Line Short Circuit Current", "", str(round(1.732*Isc, 3)), "A"])
        mat.append(["Short Circuit p.f.", "cos(phi_sc)", str(round(pf_sc, 3)), "lag"])
        mat.append(["PERFORMANCE", "", str(""), ""])
        mat.append(["At full load: Losses", "", str(round(loss_fl, 1)), "W"])
        mat.append(["At full load: Output", "", str(round(1000*rating.power_in_kw, 1)), "W"])
        mat.append(["At full load: Input", "", str(round(1000*rating.power_in_kw + loss_fl, 1)), "W"])
        mat.append(["At full load: Efficiency", "eta", str(round(eff, 1)), "%"])
        # mat.append(["At full load: Power Factor", "", str(round(phase_sc, 3)), "lag"])
        mat.append(["At full load: Slip", "", str(round(100*slip, 2)), "%"])
        mat.append(["Temperature Rise", "theta_m", str(round(theta, 2)), "degC"])
        # mat.append(["", "", str(), ""])
        # mat.append(["", "", str(), ""])

        # print(numpy.array(mat))
        df_out = pandas.DataFrame(mat)
        df_out.to_excel("Design Sheet.xlsx", header=False, index=False)
        print("Design sheet successfully prepared!")
    else:
        print("Design sheet failed...")
    print('===============================================================')

def show_rating():
    print("Showing currently set ratings ")
    print()
    print("Power rating in kW   : ", rating.power_in_kw)
    print("Power factor         : ", rating.power_factor)
    print("Efficiency           : ", rating.efficiency)
    print("Speed in rpm         : ", rating.rpm)
    print("Line voltage in V    : ", rating.line_voltage)
    print('===============================================================')

def set_rating():
    importlib.reload(rating)


from scipy.interpolate import interp1d
import numpy

def getOverhangLeakage(ratio, slot_leakage_factor_matrix):
    ans_ratio = 0
    ratio_cs_pp_column = slot_leakage_factor_matrix[:,0]
    slot_leakage_factor_column = slot_leakage_factor_matrix[:,1]
    if ratio <= 0.5:
        ans_ratio = 0.5
    elif ratio >= 1:
        ans_ratio = 1
    else:
        f = interp1d(ratio_cs_pp_column, slot_leakage_factor_column)
        ans_ratio = f(ratio).min()
    return ans_ratio


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from sys import path
import os
# path.insert(1, os.path.dirname(os.getcwd()))
from ..classes.metric import Length
from ..rating import line_voltage, poles, frequency, phase
from math import pi, ceil, sin

def stator_design_winding(Bav, tau, L, D, Kw, qs):
    yss = Length(0)
    
    flux_per_pole = Bav*tau*L
    print('Flux per pole:', round(flux_per_pole, 3))
    
    Ts = ceil(line_voltage/(4.44*frequency*flux_per_pole*Kw))
    print('Stator turns per phase:', round(Ts, 3))

    Ss = phase*qs*poles
    print('Stator slots:', Ss)

    yss.setLength(pi*D/Ss)
    print('Stator slot pitch:', round(yss.m, 3))

    stator_conductors = 2*phase*Ts
    print('Stator conductors:', stator_conductors)
    
    Zss = ceil(stator_conductors/Ss)
    print('Stator conductors per slot:', Zss)
    
    Cs = Ss/poles
    print('Coil span:', Cs)
    
    if (Cs%2==0):
        Kp = 0.966    #Kp=cos(60/2)
        Cs=Cs-1
        print("New Cs:",Cs)
    else:
        Kp = 1
        
    rad1 = (pi*30)/180
    rad2 = (pi*30)/(180*qs)
    Kd = sin(rad1)/(qs*sin(rad2))    
    
    Kws = Kp*Kd
    print('stator winding Factor:', Kws)
    
    return flux_per_pole, Ts, int(Ss), yss, Zss, Kws, Cs, Kp, Kd



import os
import pandas
import numpy
from scipy.interpolate import interp1d

package_dir = os.path.dirname(os.path.abspath(__file__))

lohys_bat_df = pandas.read_excel(package_dir+'/lookupTables/LohysBAT.xlsx')
lohys_bat_matrix = lohys_bat_df.values

def getATFromLohys(B):
    ans_lohys_bat = 0
    B_column = lohys_bat_matrix[:,0]
    At_column = lohys_bat_matrix[:,1]
    if B <= 0:
        ans_lohys_bat = 0
    elif B >= 2:
        ans_lohys_bat = 25000 + 9000*(B-2)/0.1
    else:
        f = interp1d(B_column, At_column)
        ans_lohys_bat = f(B).min()
    return ans_lohys_bat

print(getATFromLohys(1.2))


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ..classes.metric import Length,Area
from ..noLoadCurrent.LohysLoss import getSpecificIronLoss
from math import pi

def loss_stator_core(D0, D, dss, Li, Bcs, lohys_loss_matrix):
    V_core = pi*(pow(D0.m,2) - pow((D.m+2*dss.m), 2))*Li.m/4
    print("Volume of stator core: ", V_core)
    
    weight_core = V_core*7.6*pow(10,3)
    print("Weight of stator core: ", weight_core)
    
    print("Flux density in core: ", Bcs)
    
    sploss = getSpecificIronLoss(Bcs, lohys_loss_matrix)
    print("Iron loss per kg: ", sploss)
    
    loss_core = sploss*weight_core
    print("Iron loss in core: ", loss_core)
    
    return loss_core



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from ..classes.metric import Length, Area
from ..rating import poles
from math import pi, ceil
from ..rating import power_in_kw


def end_rings_rotor(Sr, Ib, Dr, Dsr, rotor_bar_loss):
    Ae_init = Area(0)
    Ae = Area(0)
    de = Length(0)
    tc = Length(0)
    outer_dia = Length(0)
    inner_dia = Length(0)
    mean_dia = Length(0)
    
    Ie = Sr*Ib/(pi*poles)
    print("End ring current: ",Ie)
    
    current_density = 6 #A/mm2
    
    Ae_init.setArea((Ie/current_density)*pow(10, -6))
    print("Reqd end ring Area: ", Ae_init.mm2)
    
    Ae.setArea(round(ceil(Ae_init.mm2/10)*10)*pow(10, -6))# rounding off to nearest multiple of 10
    print("Final end ring area: ", Ae.mm2)

    delta_e = (current_density*Ae_init.mm2)/Ae.mm2
    print("Actual current density of end ring: ", delta_e)    

    de.setLength(0.01)# depth of end ring constant at 10 mm
    print("Depth of ring: ", de.mm)
    
    tc.setLength((Ae.mm2/10)*pow(10, -3))
    print("Thickness of ring: ",tc.mm)

    outer_dia.setLength(Dr.m - 2*Dsr.m)
    print("Outer diameter of end ring: ", outer_dia.mm)

    inner_dia.setLength(outer_dia.m - 2*de.m)
    print("Inner diameter of end rings: ", inner_dia.mm)        
    
    mean_dia.setLength((outer_dia.m + inner_dia.m)/2)
    print("Mean diameter of end ring: ", mean_dia.mm)
    
    res_ring = 0.021*pi*mean_dia.m/Ae.mm2
    print("Resistance of each ring: ", res_ring)
    
    ring_copper_loss = 2*Ie*Ie*res_ring
    print("Copper loss in 2 end rings: ", ring_copper_loss)
    
    rotor_copper_loss = rotor_bar_loss + ring_copper_loss
    print("Total rotor copper loss: ", rotor_copper_loss)
    
    r = rotor_copper_loss/(power_in_kw*pow(10,3))

    slip = r/(1+r)
    print("Full load slip: ", slip*100)
    
    return rotor_copper_loss, slip, Ae, delta_e, mean_dia, res_ring, Ie, ring_copper_loss

    
        
    


#!/usr/bin/env python3
# -*- coding: utf-8 -*
from math import sqrt,pi,acos
from ..rating import line_voltage,power_in_kw

def Z_losses_eff(Rs,Xs,stator_copper,rotor_copper,no_load):
    Z=sqrt(pow(Rs,2)+pow(Xs,2))
    print("Total impedance of rotor at standstill:",Z)
    
    Isc=line_voltage/Z
    print("Isc:",Isc)
    
    pf_sc=Rs/Z
    print("short circuit power factor:",pf_sc)
    
    phase_sc=acos(pf_sc)*180/pi
    print("Phase angle of short circuit current:",phase_sc)
    
    loss_fl=stator_copper+rotor_copper+no_load
    print("Total loss at full load:",loss_fl)
    
    eff=100*power_in_kw*1000/(power_in_kw*1000+loss_fl)
    print("Efficiency:",eff)
    
    return Z,Isc,pf_sc,phase_sc,loss_fl,eff



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ..rating import poles

def mag_no_load(Kws, Ts, ATg, ATts, ATcs, ATcr, ATtr):
    AT60 = ATg + ATts + ATcs + ATcr + ATtr
    print("AT60: ", AT60)
    
    Im = (0.427*poles*AT60)/(Kws*Ts)
    print("No load magnetising current: ", Im)
    
    return Im, AT60



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ..classes.metric import Length,Area
from ..noLoadCurrent.LohysBAt import getATFromLohys
from math import pi
from ..rating import poles

def mag_stator_core(D, Li, dcs, dss, Bcs, lohys_bat_matrix):
    Acs = Area(0)
    lcs = Length(0)
    
    Acs.setArea(Li.m*dcs.m)
    print("Area of stator core: ", Acs.m2)
    
    print("Flux density in stator core: ", Bcs)
    
    lcs.setLength(pi*(D.m + 2*dss.m + 2*dcs.m)/(3*poles))
    print("Length of magnetic path through stator core: ", lcs.m)
    
    alpha = getATFromLohys(Bcs, lohys_bat_matrix)
    print("alphatcs: ", alpha)
    
    ATcs = alpha*lcs.m
    print("Mmf required for stator core: ", ATcs)
    
    return ATcs



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from ..rating import power_factor

def resistance(Ts,As,Lmts,Is,rotor_copper_loss):
    rs=0.021*Ts*Lmts.m/As.mm2
    print("Resistance of stator winding per phase:",rs)
    
    stator_copper_loss=3*Is*Is*rs
    print("Total stator copper loss:",stator_copper_loss)
    print("Total rotor copper loss:",rotor_copper_loss)
    print("Rotor copper loss per phase:",rotor_copper_loss/3)
    
    Rr=rotor_copper_loss/(Is*Is*pow(power_factor,2))
    print("Rotor resistance referred to stator:",Rr)
    
    Rs=rs+Rr
    print("Total resistance referred to stator Rs:",Rs)
    
    return Rs, stator_copper_loss, rs, Rr


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from math import pi
from ..classes.metric import Length,Area
from ..rating import ns

def temperature_rise(iron_loss,Pc,L,Lmts,D,D0,temp):
    out_surface=Area(0)
    in_surface=Area(0)
    both_ends_surface=Area(0)
    
    slot_copper_loss=2*L.m*Pc/Lmts.m
    print("Copper loss in slot portion:",slot_copper_loss)
    
    total1=iron_loss+slot_copper_loss
    print("Total loss to be dissipated by stator surface:",total1)
    
    out_surface.setArea(pi*D0.m*L.m)
    print("Outside cylindrical surface of stator:",out_surface.m2)
    
    in_surface.setArea(pi*D.m*L.m)
    print("Inner cylindrical surface area of stator:",in_surface.m2)
    
    Va=pi*D.m*ns
    print("Peripheral speed:",Va)
    
    both_ends_surface.setArea(pi*(pow(D0.m,2)-pow(D.m,2))/2)
    print("Cooling surface area of both ends:",both_ends_surface.m2)
    
    Vair=0.1*Va
    print("Velocity of air at end surface:",Vair)
    
    C1=[0.025, 0.030, 0.035, 0.04]
    C2=[0.03, 0.04, 0.05]
    C3=[0.08, 0.1, 0.15, 0.2]
    
    i=0#variable for iteratine over colling coefficient
    flag=1
    
    while(flag==1):
        print("Iteration:",(i))
        
        loss_back_core=out_surface.m2/C1[i]
        print("Loss dissipated from back of core:",loss_back_core)
        
        cool_coeff_inside=C2[i]/(1+0.1*Va)
        print("Cooling coefficient:", cool_coeff_inside)
        
        loss_inside_stator=in_surface.m2/cool_coeff_inside
        print("Loss dissipated from inside surface of stator:",loss_inside_stator)
        
        cool_coeff_end=C3[i]/Vair
        print("Cooling Coefficient:",cool_coeff_end)
        
        loss_end=both_ends_surface.m2/cool_coeff_end
        print("Loss dissipated from end surface:",loss_end)
        
        total2=loss_back_core+loss_end+loss_inside_stator
        print("Total loss dissipated:",total2)
        
        theta=total1/total2
        print("Temperature rise:",theta)
        
        if(theta<temp):
            flag=0
        else:
            i = i+1
    return theta
        
        
        
        


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ..classes.metric import Length,Area
from math import pi

def rotor_slot_leakage(Ws,W0,h1,h3,h4,Kws,Ss,Sr):
    sp_slot_per=4*pi*pow(10,-7)*(2*h1.m/(3*Ws.m)+2*h3.m/(Ws.m+W0.m)+h4.m/W0.m)
    print("Specific slot permeance for rotor:",sp_slot_per)
    
    slot_perm_stator=sp_slot_per*Kws*Kws*Ss/Sr
    print("Referred to stator side:",slot_perm_stator)
    
    return slot_perm_stator


import pandas
import os

def load_table():
    package_directory = os.path.dirname(os.path.abspath(__file__))

    df_swg = pandas.read_excel(package_directory + '/lookupTables/SWG.xlsx')
    rect_swg = pandas.read_excel(package_directory + '/lookupTables/RectangularCopperConductor.xlsx', header=None)
    carter_coeff = pandas.read_excel(package_directory + '/lookupTables/Carter.xlsx', header=None)
    lohys_stamping = pandas.read_excel(package_directory + '/lookupTables/LohysBAT.xlsx', header=None)
    fwloss = pandas.read_excel(package_directory + '/lookupTables/fwloss.xlsx', header=None)
    lohys_loss = pandas.read_excel(package_directory + '/lookupTables/LohysLoss.xlsx', header=None)
    air_gap = pandas.read_excel(package_directory + '/lookupTables/AirGap.xlsx')
    slot_leakage = pandas.read_excel(package_directory + '/lookupTables/SlotLeakageTable.xlsx', header=None)

    swg_matrix = df_swg.values
    rect_matrix = rect_swg.values
    carter_matrix = carter_coeff.values
    lohys_bat_matrix = lohys_stamping.values
    fwloss_matrix = fwloss.values
    lohys_loss_matrix = lohys_loss.values
    air_gap_matrix = air_gap.values
    slot_leakage_matrix = slot_leakage.values

    print("Running...")
    return swg_matrix, rect_matrix, carter_matrix, lohys_bat_matrix, fwloss_matrix, lohys_loss_matrix, air_gap_matrix, slot_leakage_matrix

# s,t,u,v,w,x,y = load_table()
# df_swg = pandas.read_excel('os.getcwd()+/lookupTables/SWG.xlsx')


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from math import pi
from ..rating import frequency,poles
from ..classes.metric import Length

def slot_leakage_reactance(stator_perm,rotor_perm,Ts,L,qs):
    total_slot_perm=stator_perm+rotor_perm
    print("Total specific slot permeance:",total_slot_perm)
    
    xs=8*pi*frequency*Ts*Ts*total_slot_perm/(poles*qs)
    print("Slot leakage reactance xs(ohm):",xs)
    
    return xs


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from ..rating import power_in_kw, line_voltage
from ..noLoadCurrent.frictionWindageLoss import getFrictionWindageLoss
from math import sqrt, acos, pi

def no_load_current(loss_st, loss_sc, Im, Is, fwloss_matrix):
    iron_loss = 2*(loss_st + loss_sc)
    print("Total iron loss: ", iron_loss)
    
    fw_loss_percent = getFrictionWindageLoss(power_in_kw, fwloss_matrix)
    fw_loss = fw_loss_percent*power_in_kw*pow(10, 3)/100
    print("F and W loss: ", fw_loss)
    
    no_load_loss = iron_loss + fw_loss
    print("Total no load losses: ", no_load_loss)
    
    Il = no_load_loss/(3*line_voltage)
    print("Loss component of no load current per phase Il: ", Il)
    
    I0 = sqrt(pow(Im,2) + pow(Il,2))
    print("No load current: ", I0)
    
    I0_percent_Is = I0*100/Is
    print("No load current as percentage of full load current: ", I0_percent_Is)
    
    if(I0_percent_Is > 50):
        currentCriterion = False
    else:
        currentCriterion = True
    
    no_load_pf = Il/I0
    print("No load power factor: ", no_load_pf)
    
    phase_angle = acos(no_load_pf)*180/pi
    print("Phase angle of no load current: ", phase_angle)
    print("Current Criterion: ", currentCriterion)
    return I0, phase_angle, currentCriterion, no_load_loss, fw_loss, iron_loss, Il, no_load_pf



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from ..classes.metric import Length,Area
from math import ceil

def stator_core(flux_per_pole, Li, D, dss):
    Acs = Area(0)
    dcs_init = Length(0)
    dcs = Length(0)
    stator_outer_D0 = Length(0)
             
    flux_core = flux_per_pole/2
    
    flux_density = 1.2 #assumption
    
    Acs.setArea(flux_core/flux_density)
    print("Area of stator core: ", Acs.m2)
    
    dcs_init.setLength(Acs.m2/Li.m)
    print("Depth of core: ", dcs_init.mm)
    
    dcs.setLength(ceil(dcs_init.mm)*pow(10, -3))
    
    flux_density_core_actual = (dcs_init.mm*1.2)/dcs.mm
    print("Flux density in stator core: ", flux_density_core_actual)
    
    stator_outer_D0.setLength(D.m + 2*dss.m + 2*dcs.m)
    print("Outer diameter of stator laminations: ", stator_outer_D0.mm)
    
    return Acs, dcs, stator_outer_D0, flux_density_core_actual
    


# Anwesa
# power_in_kw = 20
# power_factor = 0.82
# efficiency = 0.85
# rpm = 1000
# line_voltage = 415

# Pranabendra
# power_in_kw = 2.98
# power_factor = 0.84
# efficiency = 0.85
# rpm = 1500
# line_voltage = 415

# power_in_kw = 8
# power_factor = 0.83
# efficiency = 0.8
# rpm = 1500
# line_voltage = 415

# power_in_kw = 5.22
# power_factor = 0.83
# efficiency = 0.83
# rpm = 1000
# line_voltage = 415

# power_in_kw = 3
# power_factor = 0.83
# efficiency = 0.8
# rpm = 1500
# line_voltage = 415

# power_in_kw = 30
# power_factor = 0.85
# efficiency = 0.85
# rpm = 1500
# line_voltage = 110

# power_in_kw = 8.95
# power_factor = 0.83
# efficiency = 0.8
# rpm = 1500
# line_voltage = 415

# power_in_kw = 15
# power_factor = 0.82
# efficiency = 0.85
# rpm = 1500
# line_voltage = 415

####################
# Bodhisatya
# power_in_kw = 25
# power_factor = 0.85
# efficiency = 0.85
# rpm = 3000
# line_voltage = 415

# # Vishal
# power_in_kw = 5
# power_factor = 0.83
# efficiency = 0.8
# rpm = 1500
# line_voltage = 400

# # Abhrajit
# power_in_kw = 37.3
# power_factor = 0.85
# efficiency = 0.85
# rpm = 1000
# line_voltage = 415

# Jayanta
# power_in_kw = 8.95
# power_factor = 0.83
# efficiency = 0.8
# rpm = 1000
# line_voltage = 110

# Ramij
# power_in_kw = 50
# power_factor = 0.85
# efficiency = 0.85
# rpm = 1000
# line_voltage = 415

# Sayan
# power_in_kw = 8
# power_factor = 0.83
# efficiency = 0.8
# rpm = 1000
# line_voltage = 415

print("Enter the following ratings below")
print()
power_in_kw = float(input(  "Enter power rating in kW   : "))
power_factor = float(input( "Enter power factor         : "))
efficiency = float(input(   "Enter efficiency           : "))
rpm = float(input(          "Enter speed in rpm         : "))
line_voltage = float(input( "Enter line voltage in V    : "))


Kw = 0.95

phase = 3
frequency = 50

ns = rpm/60
poles = (2*frequency)/ns

duct_exceed_length = 120/1000
duct_width = 10/1000


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from math import pi
from ..rating import poles, frequency
from ..shortCircuitCurrent.interpolateLeakage import getOverhangLeakage

def overhang_leakage(yss,tau,Cs,stator_slots,Ts,qs,slot_leakage_matrix):
    pole_pitch = stator_slots/poles
    ratio = Cs/pole_pitch
    print("Ratio: ",ratio)
    
    Ks = getOverhangLeakage(ratio, slot_leakage_matrix)
    print("Ks: ",Ks)
    
    L_lambda = 4*pi*pow(10,-7)*Ks*tau.m*tau.m/(pi*yss.m)
    print("Product L_lamba: ", L_lambda)
    
    X0 = 8*pi*frequency*Ts*Ts*L_lambda/(poles*qs)
    print("Overhang Leakage reactance X0: ", X0)
    
    return X0



import os
import pandas
import numpy
from scipy.interpolate import interp1d

package_dir = os.path.dirname(os.path.abspath(__file__))

fwloss_df = pandas.read_excel(package_dir+'/lookupTables/fwloss.xlsx', header=None)
fwloss_matrix = fwloss_df.values

def getFrictionWindageLoss(power_rating):
    ans_fwloss = 0
    power_column = fwloss_matrix[:,0]
    perc_column = fwloss_matrix[:,1]
    if power_rating <= 0.75:
        ans_fwloss = 5.5
    elif power_rating >= 150:
        ans_fwloss = 1
    else:
        f = interp1d(power_column, perc_column)
        ans_fwloss = f(power_rating).min()
    return ans_fwloss

print(getFrictionWindageLoss(2.98))


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import pandas
from ..classes.metric import Area, Length
from math import ceil, floor, pi, sqrt, exp
from ..rating import line_voltage, power_factor, efficiency, power_in_kw
from ..statorDesign.selectConductor import select_conductor

def stator_conductor_size_and_slot(flux_per_pole, Ss, poles, D, L, Li, tau, delta, Zss, space_factor, rectangular_excel_file, round_excel_file):
    def factorize(n):
        gm = sqrt(n)
        if gm == floor(gm):
            return gm, gm
        else:
            gm = floor(gm)
            while gm > 0:
                if n%gm == 0:
                    break
                gm -= 1
            return max(gm, n//gm), min(gm, n//gm)

    As = Area(0)
    space_conductors = Area(0)
    slot_area =  Area(0)
    stator_teeth_minm = Length(0)
    stator_teeth_actual = Length(0)
    slot_width_inner_W2 = Length(0)
    slot_width_outer_Ws = Length(0)
    slot_height_h1 = Length(0)
    dss = Length(0)
    Lmts = Length(0)
    lip_h4 = Length(0)
    wedge_h3 = Length(0)
    slot_opening_W0 = Length(0)
    
    Is = (power_in_kw*pow(10, 3))/(3*line_voltage*efficiency*power_factor)
    print('Stator current per phase:', Is)
    
    #delta is in A/mm^2
    As.setArea((Is*pow(10, -6)/delta))
    print('Stator conductor required area:', As.mm2)
    
    i = 0 #variable to be used later for iterating over areas for rectangular conductor
    
    if (As.mm2 > 10.5):
        isCircular = False
        all_As, all_dim = select_conductor(As.mm2, rectangular_excel_file, isCircular)
        # print(all_As, all_dim)
        As.setArea(all_As[i]*pow(10,-6))
        a,b = all_dim[i]
        ### Change the output variables
    else:
        isCircular = True
        _, new_As, d = select_conductor(As.mm2, round_excel_file, isCircular) # swg, new_As, d
        As.setArea(new_As*pow(10, -6))
        ### Change the output variables
        
    delta_s = Is/As.mm2
    print('Area of conductor needed: ', As.mm2)
    print('Actual current density(A/mm2): ', delta_s)
    
    space_conductors.setArea(Zss*As.m2)
    print('Space for bare conductors(mm2): ', space_conductors.mm2)
    
    slot_area.setArea(space_conductors.m2/space_factor) #used only for tapered slot
    print('Area of each slot(mm2): ', slot_area.mm2)
    
    stator_teeth_minm.setLength((flux_per_pole*poles)/(1.7*Ss*Li.m))
    print('Minimum stator teeth width: ', stator_teeth_minm.mm)
    
    func = 2 + stator_teeth_minm.mm - 2*exp(-stator_teeth_minm.mm/2)

    stator_teeth_actual.setLength((ceil(func))*pow(10, -3))
    print('Actual minimum tooth width: ', stator_teeth_actual.mm)
    
    
    if isCircular:
        lip_h4.setLength(0.001)
        wedge_h3.setLength(0.003)
        lip_wedge = lip_h4.mm+wedge_h3.mm
        
        slot_opening_W0.setLength(0.002)
        
        slot_width_inner_W2.setLength(((pi*(D.mm + 2*lip_wedge))/Ss - stator_teeth_actual.mm)*pow(10, -3))
        print("Inner slot width: ", slot_width_inner_W2.mm)
        a = pi/Ss
        b = slot_width_inner_W2.mm
        c = -slot_area.mm2
        slot_height_h1.setLength(max((-sqrt(pow(b,2)-4*a*c)-b)/(2*a)*pow(10, -3), (sqrt(pow(b,2)-4*a*c)-b)/(2*a)*pow(10, -3)))
        print('Slot height: ', slot_height_h1.mm)
        
        slot_width_outer_Ws.setLength(slot_width_inner_W2.m + (2*pi*slot_height_h1.m)/Ss)
        
        dss.setLength(slot_height_h1.m + lip_wedge*pow(10,-3))
        
    else:
        lip_h4.setLength(0.001)
        wedge_h3.setLength(0.003)
        lip_wedge = lip_h4.mm+wedge_h3.mm
        slot_width_inner_W2.setLength(((pi*(D.mm + 2*lip_wedge))/Ss - stator_teeth_actual.mm)*pow(10, -3))
        slot_width_outer_Ws.setLength(slot_width_inner_W2.m)
        z1 = int(slot_width_inner_W2.mm/a)
        z2 = int(slot_width_inner_W2.mm/b)

        print('z1: ', z1)
        print('z2: ', z2)
        print('a: ', a)
        print('b: ', b)
        
        lip_h4.setLength(0.001)
        wedge_h3.setLength(0.003)
        lip_wedge_slack = lip_h4.mm + wedge_h3.mm + 2
        
        slot_opening_W0.setLength(0.003)
        
        if(Zss%z1 == 0 and Zss%z2 != 0):
            z = Zss/z1
            slot_height_h1.setLength(z*b*pow(10, -3))
            dss.setLength((z*b + lip_wedge_slack + 1.5)*pow(10, -3))
            print('Entered 1')
        elif(Zss%z2 == 0 and Zss%z1 != 0):
            z = Zss/z2
            slot_height_h1.setLength(z*a*pow(10,-3))
            dss.setLength((z*a + lip_wedge_slack + 1.5)*pow(10, -3))
            print('Entered 2')
        elif(Zss%z1 == 0 and Zss%z2 == 0):
            if z1>z2:
                z = Zss/z1
                slot_height_h1.setLength(z*b*pow(10,-3))
                dss.setLength((z*b + lip_wedge_slack + 1.5)*pow(10, -3))
                print('Entered 3')
            else:
                z = Zss/z2
                slot_height_h1.setLength(z*a*pow(10, -3))
                dss.setLength((z*a + lip_wedge_slack + 1.5)*pow(10, -3))
                print('Entered 4')
        else:
            f1, f2 = factorize(Zss)
            print(f1, f2)
            if f1 <= z1 and f2 <= z2:
                slot_height_h1.setLength(f2*b*pow(10,-3))
                dss.setLength((f2*b + lip_wedge_slack + 1.5)*pow(10, -3))
                print('Entered 5')
            elif f2 <= z1 and f1 <= z2:
                slot_height_h1.setLength(f1*b*pow(10,-3))
                dss.setLength((f1*b + lip_wedge_slack + 1.5)*pow(10, -3))
                print('Entered 6')
            else:
                print('Mayday...')
    print('Slot Dimensions: ')
    
    if isCircular:
        print('Slot Type: Tapered')
        print('Slot Width Inner(at AA): ', slot_width_inner_W2.mm)
        print('slot width Outer(at bottom): ', slot_width_outer_Ws.mm)
        print('Depth of slot(in mm): ', dss.mm)
    else:
        print('Slot Type: Parallel')
        print('Slot Width(at AA): ', slot_width_inner_W2.mm)
        print('Depth of slot(in mm): ', dss.mm)
    
    flux_density_stator_teeth = (flux_per_pole*poles)/(Ss*stator_teeth_actual.m*Li.m)
    print('Actual flux density in stator teeth: ', flux_density_stator_teeth)
    
    Lmts.setLength((2*L.m + 2.3*tau.m + 0.24))
    print('Length of mean turn: ', Lmts.m)
    
    if isCircular:
        a = d
        b = d
    return Is, slot_width_inner_W2, slot_width_outer_Ws, dss, stator_teeth_actual, Lmts, isCircular, lip_h4, wedge_h3, slot_height_h1, flux_density_stator_teeth, As, a, b, delta_s


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ..classes.metric import Length, Area
from math import pi
from ..rating import power_factor, poles, power_in_kw
from ..rotorDesign.selectConductor import select_conductor

def rotor_slots(Ss, Kws, Ts, Is, Dr, rect_mat, flux_per_pole, Li, L):
    rotor_slot_pitch = Length(0)
    rotor_bar_area = Area(0)
    Wsr = Length(0)
    Dsr = Length(0)
    slot_pitch_bottom = Length(0)
    Wt = Length(0)
    Wt_13 = Length(0)
    At_13 = Area(0)
    Lb = Length(0)
    Ws = Length(0)
    W0 = Length(0)
    h1 = Length(0)
    h3 = Length(0)
    h4 = Length(0)
    
    if poles == 6:
        Sr = Ss - 3
    else:
        Sr = Ss - 7
    print("No. of rotor slots: ", Sr)
    
    rotor_slot_pitch.setLength(pi*Dr.m/Sr)
    print("Rotor slot pitch: ", rotor_slot_pitch.mm)
    
    Ib = (2*3*Kws*Ts*Is*power_factor)/Sr
    print("Rotor bar current: ",Ib)
    
    rotor_current_density = 6 #in A/mm2
    
    rotor_bar_area.setArea((Ib/rotor_current_density)*pow(10, -6))
    print("Rotor bar area: ", rotor_bar_area.mm2)
    old_rotor_bar_area = rotor_bar_area.mm2
    
    all_areas, all_dims = select_conductor(rotor_bar_area.mm2, rect_mat, 0)
    
    i = 0#variable to be used later for iterating 
    flag = 1
    n = len(all_areas)

    if n==0:
        all_areas = [249, 249]
        all_dims = [(10, 25), (25, 10)]
        n = 2
    
    while(flag==1 and i<n):
        
        rotor_bar_area.setArea(all_areas[i]*pow(10, -6))
        print("Rotor bar area from table: ", rotor_bar_area.mm2)
    
        a, b = all_dims[i]
        
        if(b>a):#set depth to the larger value
            if power_in_kw < 10:
                Wsr.setLength((a+0.3)*pow(10,-3))
                Dsr.setLength((b+2.3)*pow(10,-3))
            elif 10<=power_in_kw<=50:
                Wsr.setLength((a+1)*pow(10,-3))
                Dsr.setLength((b+3)*pow(10,-3))
            elif power_in_kw > 50:
                Wsr.setLength((a+2)*pow(10,-3))
                Dsr.setLength((b+4)*pow(10,-3))
        else:
            if power_in_kw < 10:
                Wsr.setLength((b+0.3)*pow(10,-3))
                Dsr.setLength((a+2.3)*pow(10,-3))
            elif 10<=power_in_kw<=50:
                Wsr.setLength((b+1)*pow(10,-3))
                Dsr.setLength((a+3)*pow(10,-3))
            elif power_in_kw > 50:
                Wsr.setLength((b+2)*pow(10,-3))
                Dsr.setLength((a+4)*pow(10,-3))
    

        slot_pitch_bottom.setLength(pi*(Dr.m - 2*Dsr.m)/Sr)
        print("Slot pitch at bottom of slots: ", slot_pitch_bottom.mm)
    
        Wt.setLength(slot_pitch_bottom.m - Wsr.m)
        print("Tooth width at root: ", Wt.mm)
    
        flux_density_root = flux_per_pole*poles/(Sr*Li.m*Wt.m)
        print("Flux density at root of teeth: ", flux_density_root)
    
        if(flux_density_root<0.9):# flux density should not be a very low value
            t = Wsr.m
            Wsr.setLength(Dsr.m)
            Dsr.setLength(t)
        
        Wt_13.setLength((pi*(Dr.m - 4*Dsr.m/3)/Sr) - Wsr.m)
        print("Width of tooth at 1/3 height from narrow end: ", Wt_13.mm)
        print("Dsr: ", Dsr.mm)
        At_13.setArea((Sr*Wt_13.m*Li.m)/poles)
        print("Area of teeth at 1/3rd from narrow end: ", At_13.mm2)
        
        flux_density_13 = flux_per_pole/At_13.m2
        print("Flux density of rotor teeth at 1/3 from narrow end: ", flux_density_13)
        
        if (flux_density_13 > 1.25):  #1.7/1.36=1.25(1.25 is later used for no load calculation)
            flag = 1
            i = i+1
        else:
            flag = 0
       
    if(i>=n):
        noAreaFound=True
    else:
        noAreaFound=False
    
    Lb.setLength(L.m + 2*0.015 + 0.01)
    print("Length of each rotor bar: ", Lb.mm)
    
    R_rotor_bar = 0.021*Lb.m/rotor_bar_area.mm2
    print("Resistance of each bar(ohm):", R_rotor_bar)
    
    copper_loss = Sr*Ib*Ib*R_rotor_bar
    print("Total copper loss in bars(W): ", copper_loss)
    
    if(power_in_kw<10):
        W0.setLength(0.0015)
        Ws.setLength(Wsr.m)
        h1.setLength(Dsr.m)
        h3.setLength(0.001)
        h4.setLength(0.001)
    elif(10<=power_in_kw<=50):
        W0.setLength(0.002)
        Ws.setLength(Wsr.m)
        h1.setLength(Dsr.m)
        h3.setLength(0.0015)
        h4.setLength(0.0005)
    else:
        W0.setLength(0.002)
        Ws.setLength(Wsr.m)
        h1.setLength(Dsr.m)
        h3.setLength(0.003)
        h4.setLength(0.001)
    
    delta_r = (rotor_current_density*old_rotor_bar_area)/rotor_bar_area.mm2
    return Sr, Ib, rotor_bar_area, Wsr, Dsr, flux_density_13, copper_loss, Lb, rotor_slot_pitch, W0, Ws, h1, h3, h4, noAreaFound, delta_r, R_rotor_bar, a, b



from scipy.interpolate import interp1d
import numpy

def getFrictionWindageLoss(power_rating, fwloss_matrix):
    ans_fwloss = 0
    power_column = fwloss_matrix[:,0]
    perc_column = fwloss_matrix[:,1]
    if power_rating <= 0.75:
        ans_fwloss = 5.5
    elif power_rating >= 150:
        ans_fwloss = 1
    else:
        f = interp1d(power_column, perc_column)
        ans_fwloss = f(power_rating).min()
    return ans_fwloss



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ..classes.metric import Length, Area
from math import pi
from ..noLoadCurrent.LohysLoss import getSpecificIronLoss

def loss_stator_teeth(Ss, Wts, Li, dss, Bts, lohys_loss_matrix):
    V_st = Ss*Wts.m*Li.m*dss.m
    print("Volume of stator teeth: ", V_st)
    
    weight_st = V_st*7.6*pow(10,3)
    print("Weight of stator teeth: ", weight_st)
    
    Bm = pi*Bts/2
    print("Maximum flux density on stator teeth: ", Bm)
    
    sploss = getSpecificIronLoss(Bm, lohys_loss_matrix)
    print("Specific Iron loss: ", sploss)
    
    loss_st = sploss*weight_st
    print("Iron loss in stator teeth: ", loss_st)
    
    return loss_st
    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from ..rating import poles

def stator_teeth_actual(flux_per_pole, Ss, Wts, Li):
    flux_density_actual = flux_per_pole*poles/(Ss*Wts.m*Li.m)
    return flux_density_actual
    #Wb/m^2


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from ..noLoadCurrent.CarterCoeff import getCarterCoeff
from ..classes.metric import Length, Area
from math import pi
from ..rating import poles

def mag_air_gap(s_W0, r_W0, lg, yss, ysr, isDuct, duct_width_given, L, D, n_ducts, Bav, carter_matrix):
    Ag = Area(0)
    lge = Length(0)
    duct_width = Length(duct_width_given)
    
    r1 = s_W0.m/lg.m
    print("Ratio for stator: ", r1)
    
    Kcs = getCarterCoeff('semiClosed', r1, carter_matrix)
    print("Carter's coefficient for stator slots: ", Kcs)
    
    Kgss = yss.mm/(yss.mm - Kcs*s_W0.mm)
    print("Gap contraction factor for stator slots: ", Kgss)
    
    r2 = r_W0.m/lg.m
    print("Ratio for rotor: ", r2)
    
    Kcr = getCarterCoeff('semiClosed', r2, carter_matrix)
    print("Carter's coefficient for rotor slots: ", Kcr)
    
    Kgsr = ysr.mm/(ysr.mm - Kcr*r_W0.mm)
    print("Gap contraction factor for rotor slots: ", Kgsr)
    
    Kgs = Kgss*Kgsr
    print("Kgs: ", Kgs)
    
    if isDuct:
        r3 = duct_width.m/lg.m
        print("Ratio for duct: ", r3)
        
        Kcd = getCarterCoeff('open', r3, carter_matrix)
        print("Carter's coefficient for ducts: ", Kcd)
        
        Kgd = L.mm/(L.mm - Kcd*n_ducts*duct_width.mm)
        print("Gap contraction factor for ducts: ", Kgd)
        
    else:
        Kgd = 1
        
    Kg = Kgs*Kgd
    print("Total gap contraction factor: ", Kg)
        
    Ag.setArea(pi*D.m*L.m/poles)
    print("Area of air gap: ", Ag.m2)
    
    Bg60 = 1.36*Bav
    print("Bg60: ", Bg60)
    
    lge.setLength(Kg*lg.m)
    print("Effective length of air gap: ", lge.mm)
    
    ATg = 800000*Bg60*lge.m
    print("Mmf required for air gap: ", ATg)
    
    return ATg


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from ..rating import line_voltage,poles

def total_leakage(Im,Ss,Sr,xs,X0):
    Xm=line_voltage/Im
    print("Magnetizing reactance Xm:",Xm)
    
    qs=Ss/(3*poles)
    qr=Sr/(3*poles)
    
    Xz=(5*Xm*(1/pow(qs,2)+1/pow(qr,2)))/(6*9)#ms=3
    print("Xz:",Xz)
    
    XS =xs+X0+Xz
    print("Total leakage reactance per phase referred to stator:",XS)
    
    return Xm, XS, Xz
    


import bisect
import math

def select_conductor(area_given_by_user, matrix_conductor, isCircular):
    if isCircular:
        swg_column = matrix_conductor[:,0]
        diameter_column = matrix_conductor[:,1]
        area_column = matrix_conductor[:,2]

        swg_index = bisect.bisect_right(area_column, area_given_by_user)

        swg_number = swg_column[swg_index]
        area_to_be_used = area_column[swg_index]
        diameter_to_be_used = diameter_column[swg_index]

        return swg_number, area_to_be_used, diameter_to_be_used 
    else:
        all_possible_areas = []
        all_possible_dimensions = []
        for width_index in range(1, 33):
            for thickness_index in range(1, 31):
                if matrix_conductor[width_index][thickness_index] == -1:
                    continue
                else:
                    if matrix_conductor[width_index][thickness_index] > area_given_by_user:
                        all_possible_areas.append(matrix_conductor[width_index][thickness_index])
                        all_possible_areas.append(matrix_conductor[width_index][thickness_index])
                        all_possible_dimensions.append((matrix_conductor[width_index][0], matrix_conductor[0][thickness_index]))
                        all_possible_dimensions.append((matrix_conductor[0][thickness_index], matrix_conductor[width_index][0]))
                        break
        
        return all_possible_areas, all_possible_dimensions



import os
import pandas
import numpy
from scipy.interpolate import interp1d

package_dir = os.path.dirname(os.path.abspath(__file__))

lohys_loss_df = pandas.read_excel(package_dir+'/lookupTables/LohysLoss.xlsx')
lohys_loss_matrix = lohys_loss_df.values

def getSpecificIronLoss(B):
    ans_lohys_loss = 0
    B_column = lohys_loss_matrix[:,0]
    specific_column = lohys_loss_matrix[:,1]
    if B <= 0:
        ans_lohys_loss = 0
    elif B >= 1.8:
        ans_lohys_loss = 13 + 3.7*(B-1.8)/0.2
    else:
        f = interp1d(B_column, specific_column)
        ans_lohys_loss = f(B).min()
    return ans_lohys_loss

print(getSpecificIronLoss(2))


from scipy.interpolate import interp1d
import numpy

def getATFromLohys(B, lohys_bat_matrix):
    ans_lohys_bat = 0
    B_column = lohys_bat_matrix[:,0]
    At_column = lohys_bat_matrix[:,1]
    if B <= 0:
        ans_lohys_bat = 0
    elif B >= 2:
        ans_lohys_bat = 25000 + 9000*(B-2)/0.1
    else:
        f = interp1d(B_column, At_column)
        ans_lohys_bat = f(B).min()
    return ans_lohys_bat


from setuptools import setup, find_packages
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
      long_description = f.read()

classifiers = [
    'Intended Audience :: Education',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 3',
]

setup(name='threephasescim',
      version='0.1.0',

      packages=find_packages(where='threephasescim'),
      package_dir={'': 'threephasescim'},
      package_data={'threephasescim/threephasescim/lookupTables' : ['*.xlsx']},
      
      description='An attempt to design three phase squirrel cage induction motors using Python 3',
      url='https://github.com/pranabendra/three-phase-scim',
      author='Pranabendra Prasad Chandra',
      author_email='pranabendrachandra@gmail.com',
      license='MIT',
      classifiers=classifiers,
      include_package_data=True,
      long_description=long_description,
      long_description_content_type='text/markdown',
      install_requires=['pandas', 'numpy', 'scipy', 'xlrd', 'openpyxl',],
      zip_safe=False)



from ..rating import frequency, phase, line_voltage

def adjust_initial_values(Zss, Ss, qs, old_Bav, ac, tau, L, Kw):
    Zss = round(Zss/4)*4
    stator_conductors = Zss*Ss
    Ts = round(stator_conductors/(qs*phase))
    flux_per_pole = line_voltage/(4.44*Ts*frequency*Kw)
    new_Bav = flux_per_pole/(tau.m*L.m)
    ac = round((old_Bav*ac)/new_Bav)
    print("New Zss: ", Zss)
    print("New Ts: ", Ts)
    print("New flux per pole: ", flux_per_pole)
    print("New Bav: ", new_Bav)
    print("New ac: ", ac)

    return flux_per_pole, Ts, Ss, Zss, new_Bav, ac


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from ..classes.metric import Length
from ..rotorDesign.airGapLength import getAirGapLength
from math import sqrt

def air_gap(L, D, air_gap_matrix):
    lg = Length(0)
    Dr = Length(0)
    
    lg.setLength(round(getAirGapLength(D.m, air_gap_matrix), 2) *pow(10, -3))
    print("Airgap: ", lg.mm)
    
    Dr.setLength(D.m - 2*lg.m)
    print("Diameter of rotor: ", Dr.mm)
    
    return lg, Dr
    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ..classes.metric import Length

def rotor_core(Dr, dcs, dsr, Bcs):
    dcr = Length(0)
    Di = Length(0)
    
    dcr.setLength(dcs.m)#same as stator core
    print("Depth of rotor core(same as stator): ", dcr.mm)
    
    Bcr = Bcs
    print("Flux density in rotor core(same as stator): ",Bcr)
    
    Di.setLength(Dr.m - 2*dsr.m - 2*dcr.m)
    print("Inner diameter or rotor laminations: ", Di.mm)
    
    return dcr, Bcr, Di



# from sys import path
# path.insert(1, './classes')
from ..classes.metric import Length, Area
from ..rating import power_factor, power_in_kw, efficiency, ns, poles, duct_exceed_length, duct_width
from math import pi

def compute_main_dimension(Bav, ac, Kw, L_by_tau):
    tau = Length(0)
    D = Length(0)
    L = Length(0)
    Li = Length(0)
    duct = False

    C0 = 11*Kw*Bav*ac*pow(10, -3)
    print("C0: ", round(C0,3))

    Q = power_in_kw/(power_factor*efficiency)
    print("KVA input: ", round(Q,2))

    product_D2L = Q/(C0*ns)
    print("D2L Product: ", product_D2L)

    L_by_D = L_by_tau*pi/poles
    D.setLength(pow(product_D2L/L_by_D, 1/3))
    L.setLength(L_by_D*D.m)
    tau.setLength(L.m/L_by_tau) 

    if L.m > duct_exceed_length:
        Li.setLength(0.9*(L.m - duct_width))
        duct = True
    else:
        Li.setLength(0.9*L.m)
        duct = False

    print("L: ", round(L.m, 3))
    print("D: ", round(D.m, 3))
    print("Li: ", round(Li.m, 3))
    print("tau: ", round(tau.m, 3))
    if duct:
        print("One duct of 10mm is present")
    else:
        print("No duct present")

    print('Main Dimension Completed')
    return D, L, tau, Li, duct, C0, Q, product_D2L



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from ..noLoadCurrent.LohysBAt import getATFromLohys
from ..rating import poles
from ..classes.metric import Area

def mag_stator_teeth(Ss, Wts, Li, Bts, dss, lohys_bat_matrix):
    teeth_pole_area = Area(0)
    
    teeth_pole_area.setArea(Ss*Wts.m*Li.m/poles)
    print("Area of teeth per pole: ", teeth_pole_area.m2)
    
    print("Flux density in stator teeth(Wb/m^2): ", Bts)
    
    Bts60 = 1.36*Bts
    print("Bts60: ", Bts60)
    
    alpha = getATFromLohys(Bts60, lohys_bat_matrix)
    print("alphats60: ", alpha)
    
    ATs = alpha*dss.m
    print("Mmf required for stator teeth: ", ATs)
    
    return ATs



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from ..noLoadCurrent.LohysBAt import getATFromLohys

def mag_rotor_teeth(Btr_13, dsr, lohys_bat_matrix):
    Btr60 = 1.36*Btr_13
    print("Btr60: ", Btr60)
    
    alpha = getATFromLohys(Btr60, lohys_bat_matrix)
    print("alpha_tr: ", alpha)
    
    ATr = alpha*dsr.m
    print("Mmf required for rotor teeth: ", ATr)
    
    return ATr
